/***********************************************************************
    MULti Core Operating System - MultiLayer kenel (MULCOS-M)

          NonSecure sample program for NUCLEO-H563ZI

    Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.

    File name   main.c

    Jan/01/2024 Created.

 ***********************************************************************/

#include <string.h>


#include "kernel.h"
#include "arch_device.h"
#include "drv_uart.h"
#include "dv_stm32h56x.h"


/***********************************************************************
    Definitions of external data and function
 ***********************************************************************/

extern T_CSYS pk_csys;

extern void Led1_On(void);
extern void Led1_Off(void);
extern void Led2_On(void);
extern void Led2_Off(void);
extern void Led3_On(void);
extern void Led3_Off(void);
extern void init_mpu(void);
extern void initial_nvec(void);
extern void initial_fpu(void);
extern void start_test(void);

extern VB buff[8192];
extern VB *buff_ptr;


/***********************************************************************
    Definitions of parameter
 ***********************************************************************/

#define   CORE_CLOCK    250000000
#define   SYST_EXTNO    15
#define   PC13_EXTNO    40
#define   TIM6_EXTNO    65 /* (49+16) = 65 */

          /* Using UART3 3 */
#define   ini_sio       ini_uart3
#define   ref_sio       ref_uart3
#define   ctr_sio       ctr_uart3
#define   putc_sio      putc_uart3
#define   puts_sio      puts_uart3
#define   getc_sio      getc_uart3
#define   gets_sio      gets_uart3


/***********************************************************************
    Definitions of parameter
 ***********************************************************************/

const VB Message1[] = "The non-secure sample program for the MULCOS-MK system on NUCLEO-H563ZI (Cortex-M33).\n";
const VB Message2[] = "Detected a press on the user button.\n";


/***********************************************************************
    Definitions of prototype
 ***********************************************************************/

void default_handler(void);
void MainTask(VP_INT stacode);
void gpio_c13_int(VP_INT exinf);
void cortex_cm_systick(VP_INT exinf);
void cyc_handler(VP_INT exinf);

const T_CTSK ctsk_main = {
#ifdef PRIVILEGE
      .tskatr  = TA_HLNG|TA_PRIV|TA_ACT,
#else
      .tskatr  = TA_HLNG|TA_ACT,
#endif
      .exinf   = (VP_INT)0x5A5A5A5A,
      .task    = (FP)MainTask,
      .itskpri = 1,
      .stksz   = 0x2000,
      .stk     = 0,
      .sstksz  = 0x00000600,
      .name    = "MainTask"};
const T_DINH dinh_systick = {
      .inhatr = TA_NULL,
      .inthdr = (FP)cortex_cm_systick,
      .imask  = 0x60};
const T_CISR cisr_gpioc13 = {
      .isratr = TA_HLNG,
      .exinf  = (VP_INT)0,
      .intno  = PC13_EXTNO,
      .isr    = gpio_c13_int,
      .imask  = 0x80,
      .name   = "GpioC13" };
const T_CCYC ccyc_led = {
      .cycatr = TA_HLNG|TA_STA,
      .exinf  = (VP_INT)0x12345678,
      .cychdr = (FP)cyc_handler,
      .cyctim = 2000,
      .cycphs = 0,
      .name   = "Cyclic LED"};


/***********************************************************************
    Definitions of private data
 ***********************************************************************/

ID TskID;
ID CycID;
ID IsrID;
INT proc_switch = 0;


/***********************************************************************
 *    Default handler of abnormal interrupt
 *        for Interrupts generated by undefined interrupt factors
 ***********************************************************************/

void default_handler(void)
{
    for(;;);
}


/*******************************
        Cyclic handler
 *******************************/

void cyc_handler(VP_INT exinf)
{
    Led2_Off();
    stp_cyc(CycID);
}

/***********************************************************************
      Systick interrupt handler
 ***********************************************************************/

void cortex_cm_systick(VP exinfo)
{
    if ((SYSTICK.CSR & 0x00010000) != 0) {
        if (proc_switch == 0) {
            Led1_On();
        } else if (proc_switch == 50) {
            Led1_Off();
        } else {
            ;
        }
        if (++proc_switch == 500) {
            proc_switch = 0;
        }
    }
}


/***********************************************************************
      Initialize systick and define interrupt handler
 ***********************************************************************/

void cortex_cm_init_systick(UW interval)
{
    ER ret;

    loc_cpu();                          /* Change to Privilege */
    SYSTICK.CSR = 0x00000000;           /* Stop systick timer */

    ret = def_inh(SYST_EXTNO, (T_DINH *)&dinh_systick);
    if (ret >= E_OK) {
        SYSTICK.CVR = 0;
        SYSTICK.RVR = interval-1;
        SYSTICK.CSR = 0x00000007;
    }
    unl_cpu();                          /* Change to UnPrivilege */
}


/***********************************************************************
      User button interrupt handler
 ***********************************************************************/

void gpio_c13_int(VP_INT exinf)
{
    if ((REG_EXTI.RPR1 & 0x00002000) != 0 ) {
        Led2_On();
        sta_cyc(CycID);
        wup_tsk(TskID);
        REG_EXTI.RPR1 = 0x00002000;
    }
}


/***********************************************************************
        BUTTON pin configurations
 ***********************************************************************/

void user_button_config(void)
{

    /* Define PC13 to input port */
    REG_GPIOC.MODER  = (REG_GPIOC.MODER & ~0x0C000000) | 0x00000000;
    REG_GPIOC.PUPDR  = (REG_GPIOC.PUPDR & ~0x0C000000) | 0x00000000;
    /* Configured EXTI13 to rising trigger of PC13 */
    REG_EXTI.RSTR1  |= 0x00002000;
    REG_EXTI.EXTICR4 = (REG_EXTI.EXTICR4 & ~0x0000FF00) | 0x00000200;
    REG_EXTI.IMR1   |= 0x00002000;
    REG_EXTI.RPR1    = 0x00002000;

    IsrID = acre_isr((T_CISR *)&cisr_gpioc13);
    if (IsrID > E_OK) {
        ena_int(PC13_EXTNO);
    }
}


/*******************************
        Main Task
 *******************************/

void MainTask(__attribute__((unused)) VP_INT exinf)
{
    UINT txcnt;

    txcnt = strlen(Message1);
    puts_sio((VB *)Message1, &txcnt, TMO_FEVR);

    /* Initialize Cortex-M systick */
    cortex_cm_init_systick(CORE_CLOCK/1000);

    user_button_config();

    CycID = acre_cyc((T_CCYC *)&ccyc_led);

    for(;;) {
        slp_tsk();
        txcnt = strlen(Message2);
        puts_sio((VB *)Message2, &txcnt, TMO_FEVR);
    }
}


/***************************************
        Function of Initializing
 ***************************************/

int main(void)
{
    ER ret;

    /* Initialize NVEC */
    initial_nvec();

    /* Initialize FPU */
    initial_fpu();

    /* Initialize FPU */
    init_mpu();

    /* Initialize MULCOS-M kernel */
    ret = ini_krn(&pk_csys);
    if (ret == E_OK) {
        TskID = 1;
        ret = cre_tsk(TskID, (T_CTSK *)&ctsk_main);
        if (ret == E_OK) {
            /* Start MULCOS-M kernel */
            sta_krn();
        }
    }
    return ret;
}

/* end */
